<?php
/**
 * @package    Joomla.Site
 * @subpackage System.Gdprcccp
 *
 * @author     Yoshiki Kozaki <forgetaz@outlook.com>
 * @copyright  (C) 2007 Yoshiki Kozaki
 * @license    https://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @link     https://forgetaz.com/
 */

\defined('_JEXEC') or die;

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\HTML\HTMLHelper;

/**
 * Gdprcccp plugin.
 *
 * @package   System.Gdprcccp
 * @since     1.2.0
 */
class plgSystemGdprcccp extends CMSPlugin
{
	/**
	 * Application object
	 *
	 * @var    ?Joomla\CMS\Application\CMSApplication
	 * @since  4.0.0
	 */
	protected ?\Joomla\CMS\Application\CMSApplicationInterface $app = null;

	/**
	 * Affects constructor behavior. If true, language files will be loaded automatically.
	 *
	 * @var    boolean
	 * @since  3.1.4
	 */
	protected $autoloadLanguage = true;

	/**
	 * Cookie name
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected string $cookieName = 'cookieControl';

	/**
	 * Cookie name
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected string $cookiePrefName = 'cookieControlPrefs';

	/**
	 * CookieTypes Options
	 * @var array
	 * @since 1.0
	 */
	protected array $options = array();

	/**
	 * Values of CookieTypes
	 * @var array
	 * @since 1.0
	 */
	protected array $values = array();

	/**
	 * InlineScripts of CookieTypes
	 * @var array
	 * @since 1.0
	 */
	protected array $scripts = array();

	/**
	 * InlineScripts of CookieTypes for not accepted
	 * @var array
	 * @since 1.0
	 */
	protected array $notScripts = array();

	/**
	 * Script Files of CookieTypes
	 * @var array
	 * @since 1.0
	 */
	protected array $scriptFiles = array();

	/**
	 * Attributes of script file
	 * @var array
	 * @since 1.0
	 */
	protected array $scriptfileAttrs = array();

	/**
	 * Max number of options
	 * @var int
	 * @since 1.0
	 */
	protected int $optionsMax = 3;

	/**
	 * Change to AfterDispatch because $this->app doesn't have the document on AfterRoute
	 *
	 * @return void
	 * @since 1.5
	 */
	public function onAfterDispatch():void
	{
		if(!$this->app->isClient('site'))
		{
			return;
		}

		$doc = $this->app->getDocument();

		if($doc->getType() != 'html')
		{
			return;
		}

		//Get options
		$this->getOptions();
		//Get cookie value
		$val = $this->app->getInput()->cookie->getWord($this->cookieName);
		$val = ($val === 'true')? true:false;

		//Exists cookie
		$event = '';
		if($val)
		{
			//Get accepted values
			$pref = $this->app->getInput()->cookie->getString($this->cookiePrefName);
			$pref = \json_decode($pref);
			//Set scripts
			$this->setScripts($pref);
			//Reset event -> removed
//			return;
		}

		//Load iplist
		$filename = __DIR__. '/IpList/displays.php';
		if(\file_exists($filename))
		{
			require_once $filename;
			$ip = \Joomla\Utilities\IpHelper::getIp();
			if(\class_exists('displays') && \Joomla\Utilities\IpHelper::IPinList($ip, displays::$list))
			{
				//Set cookie values
				$expires = $this->params->get('expires', 30);
				$expirationDate = \time()+60*60*24*(int)$expires;
				$this->app->getInput()->cookie->set($this->cookieName, 'true',
					$expirationDate,
					$this->app->get('cookie_path', '/'),
					$this->app->get('cookie_domain', ''), false, false
				);
				$this->app->getInput()->cookie->set($this->cookiePrefName, \json_encode($this->values),
					$expirationDate,
					$this->app->get('cookie_path', '/'),
					$this->app->get('cookie_domain', ''), false, false
				);
				//Set all inline script
				$this->setScripts($this->values);
				//End plugin
				return;
			}
		}

		if($this->app->input->getCmd('tmpl') == 'component')
		{
			//End plugin
			return;
		}

		$title = $this->params->get('title');
		$message = $this->params->get('message');
		$useOverlay = $this->params->get('useOverlay', 'true');
		$link = $this->params->get('link');
		if($useOverlay === 'true')
		{
			$link .= (\strpos($link, '?')? '&': '?'). 'tmpl=component';
		}
		$delay = $this->params->get('delay', 2000);
		$expires = $this->params->get('expires', 30);
		$onAccept = $this->params->get('onAccept');
		$uncheckboxes = $this->params->get('uncheckBoxes', 'false');
		$moreinfoLabel = $this->params->get('moreInfoLabel');
		$acceptBtnLabel = $this->params->get('acceptBtnLabel');
		$acceptSelectedBtnLabel = $this->params->get('acceptSelectedBtnLabel');
		$advancedBtnLabel = $this->params->get('advancedBtnLabel');
		$cookieTypesTitle = $this->params->get('cookieTypesTitle');
		$fixedCookieTypeLabel = $this->params->get('fixedCookieTypeLabel');
		$fixedCookieTypeDesc = $this->params->get('fixedCookieTypeDesc');
		$useReload = $this->params->get('reload', 0);
		$showResetBtn = $this->params->get('showResetBtn', true);
		$resetBtnTitle = $this->params->get('resetBtnTitle', 'Reset cookie setting');

		//Create function
		$function = '';
		if($useReload)
		{
			$function = 'window.location.reload();';
		} else
		{
			if(\count($this->scripts) > 0)
			{
				foreach($this->scripts as $key => $value)
				{
					$function .= "if(pref.$key){ $value }";
					if(isset($this->notScripts[$key]))
					{
						$function .= "else{ {$this->notScripts[$key]} }";
					}
				}
			}

			if(!empty($function))
			{
				$function = 'var pref = $.fn.ihavecookies.cookie();'. $function;
			}
		}

		HTMLHelper::stylesheet('media/plg_system_gdprcccp/css/gdprcccp.min.css');
		HTMLHelper::_('jquery.framework');
		HTMLHelper::script('media/plg_system_gdprcccp/js/jquery.ihavecookies.min.js'
			, [], ['defer' => true]);
		//IF not use reload
		if(!$useReload)
		{
			//Register script files
			$this->setScripts($this->values, true);
		}

		$cookieTypes = ',cookieTypes:[]';
		if(\count($this->options) > 0)
		{
			$cookieTypes = ',cookieTypes:'. \json_encode($this->options);
		}

		$js = "
		(function($) {
			var options = {
		        title: '$title',
				message: '$message',
		        delay: $delay,
		        expires: $expires,
		        link: '$link',
		        onAccept: function(){ $onAccept$function },
		        uncheckBoxes: $uncheckboxes,
		        acceptBtnLabel: '$acceptBtnLabel',
		        acceptSelectedBtnLabel:'$acceptSelectedBtnLabel',
		        advancedBtnLabel:'$advancedBtnLabel',
		        moreInfoLabel: '$moreinfoLabel',
		        cookieTypesTitle: '$cookieTypesTitle',
		        fixedCookieTypeLabel: '$fixedCookieTypeLabel',
		        fixedCookieTypeDesc: '$fixedCookieTypeDesc'$cookieTypes,
		        cookieName: '$this->cookieName',
		        cookiePrefName: '$this->cookiePrefName',
		        useOverlay: $useOverlay,
		        showResetBtn: $showResetBtn,
		        resetBtnTitle: '$resetBtnTitle'
		    }
			$('body').ihavecookies(options$event);
	    })(jQuery);";
		$doc->getWebAssetManager()->addInlineScript($js, [], ['type' => 'module']);
	}

	/**
	 * Get configuration
	 *
	 * @return void
	 * @since 1.0
	 */
	protected function getOptions():void
	{
		$this->cookieName = $this->params->get('cookieName', $this->cookieName);
		$this->cookiePrefName = $this->params->get('cookiePrefName', $this->cookiePrefName);

		$max = $this->optionsMax + 1;
		for($i = 1; $i < $max; $i++)
		{
			$opn = 'use_option'. $i;
			if(!$this->params->get($opn))
			{
				continue;
			}

			$prefix = 'op'. $i;
			$value = $this->params->get($prefix. 'value', '');
			if(!empty($value))
			{
				$this->values[] = $value;

				$this->options[] = array(
					'type' => $this->params->get($prefix. 'title', ''),
					'value' => $value,
					'description' => $this->params->get($prefix. 'desc', '')
				);

				$file = $this->params->get($prefix. 'scriptfile');
				if(!empty($file))
				{
					$this->scriptFiles[$value] = $file;
					$this->scriptfileAttrs[$value] = $this->params->get($prefix. 'scriptattr');
				}

				$script = $this->params->get($prefix. 'script');
				if(!empty($script))
				{
					$this->scripts[$value] = $script;
				}

				$notscript = $this->params->get($prefix. 'notscript');
				if(!empty($notscript))
				{
					$this->notScripts[$value] = $notscript;
				}
			}
		}
	}

	/**
	 * Set script
	 *
	 * @param   array|null  $pref
	 *
	 * @return void
	 * @since 1.0
	 */
	protected function setScripts(?array $pref, bool $filesonly = false):void
	{
		$function = '';
		if(!empty($pref))
		{
			//Accept all or
			foreach($pref as $key)
			{
				if(isset($this->notScripts[$key]))
				{
					unset($this->notScripts[$key]);
				}

				if(isset($this->scriptFiles[$key]))
				{
					$attr = array();
					if(!empty($this->scriptfileAttrs[$key]))
					{
						$attr = [$this->scriptfileAttrs[$key] => true];
					}
					HTMLHelper::script($this->scriptFiles[$key], [], $attr);
				}

				if(isset($this->scripts[$key]))
				{
					$function .= $this->scripts[$key];
				}
			}
		}

		if($filesonly)
		{
			return;
		}

		if(\count($this->notScripts))
		{
			foreach ($this->notScripts as $value)
			{
				$function .= $value;
			}
		}

		if(!empty($function))
		{
			$this->app->getDocument()->getWebAssetManager()->addInlineScript(
				$function, [], ['type' => 'module']);
		}
	}
}